from flask import (
    Flask, request, redirect, make_response, render_template, send_file)
import sys
from six import text_type
import threading
import imprimatur
import traceback
from io import BytesIO
from zipfile import ZIP_DEFLATED, ZipFile


app = Flask(__name__)

procs = {}

proc_lock = threading.Lock()


class RunThread(threading.Thread):
    def __init__(self, script, **kwargs):
        threading.Thread.__init__(self, **kwargs)
        self.script = script
        self.results = []
        self.results_lock = threading.Lock()

    def run(self):
        try:
            for txt in imprimatur.run(self.script):
                with self.results_lock:
                    self.results.append(txt)

        except BaseException:
            sys.stderr.write("An error:\n")
            sys.stderr.write(traceback.format_exc())

    def results_str(self):
        with self.results_lock:
            if len(self.results) == 0:
                return "Hasn't started yet..."
            elif self.results[-1] is None:
                txts = self.results[:-1]
            else:
                txts = self.results[:]
        return ''.join(txts)


@app.route('/', methods=['GET', 'POST'])
def home():
    if request.method == 'GET':
        with proc_lock:
            runs = procs.keys()
        return render_template('home.html', runs=runs)
    else:
        fl = request.files['file']
        script = text_type(fl.stream.read(), 'utf8')
        proc = RunThread(script)
        with proc_lock:
            proc_id = len(procs)
            procs[proc_id] = proc
            proc.start()
        return redirect('/runs/' + str(proc_id), 303)


@app.route('/runs/<int:run_id>')
def runs(run_id):
    with proc_lock:
        proc = procs[run_id]
    return render_template('run.html', proc=proc, run_id=run_id)


@app.route('/text_1')
def test_1():
    response = make_response(b'\xfeThe best of all possible worlds.')
    response.headers['content-length'] = '33'
    response.headers['content-type'] = 'text; charset=utf-8'
    response.headers['date'] = 'Sat, 17 Jan 2015 13:30:54 GMT'
    response.headers['server'] = 'Werkzeug/0.9.6 Python/3.4.0'
    return response


@app.route('/text_2')
def test_2():
    return 'I warn you Mr Bond, my patience is not inexhaustible.'


@app.route('/text_2_zip')
def test_2_zip():
    zfile = BytesIO()
    with ZipFile(zfile, mode='w', compression=ZIP_DEFLATED) as zf:
        zf.writestr(
            'bond.zip',
            'I warn you Mr Bond, my patience is not inexhaustible.')
    zfile.seek(0)
    return send_file(zfile, attachment_filename='bond.zip', as_attachment=True)


@app.route('/blank')
def blank():
    response = make_response('')
    response.headers['content-length'] = '0'
    response.headers['content-type'] = 'text; charset=utf-8'
    response.headers['date'] = 'Sat, 17 Jan 2015 13:30:54 GMT'
    response.headers['server'] = 'Werkzeug/0.9.6 Python/3.4.0'
    return response


@app.route('/echo', methods=['POST'])
def echo():
    ret = []
    fls = list(request.files.items())
    if len(fls) > 0:
        fname, fval = fls[0]
        ret.append(fname + ": " + text_type(fval.stream.read(), 'utf8') + '\n')
    ret.append(str(request.form) + '\n')
    return ''.join(ret)


@app.route('/echo_bin', methods=['POST'])
def echo_bin():
    return "okay"


@app.route('/redirect')
def redir():
    location = request.args['location']
    return redirect(location)


@app.route('/auth')
def auth():
    auth = request.authorization
    if auth is not None and auth.username == 'conrad' and \
            auth.password == 'kurtz':
        response = make_response('authorized')
    else:
        response = make_response('not authorized', 403)
    response.headers['content-length'] = '14'
    response.headers['content-type'] = 'text; charset=utf-8'
    response.headers['date'] = 'Sat, 17 Jan 2015 13:30:54 GMT'
    response.headers['server'] = 'Werkzeug/0.9.6 Python/3.4.0'
    return response


count = 0


@app.route('/counter_reset')
def counter_reset():
    global count
    count = 0
    return "Counter reset.\n"


@app.route('/counter')
def counter():
    global count
    count += 1
    if count == 3:
        return "The Decline And Fall Of The Roman Empire\n"
    else:
        return str(count) + "Gibbon\n"


@app.errorhandler(500)
def page_not_found(e):
    error = traceback.format_exc()
    sys.stderr.write(error)
    return error, 500


if __name__ == '__main__':
        app.run(debug=True)



import requests
import re
import time
from six import text_type
import traceback
import sre_constants
import zipfile
from io import BytesIO


def response_str(response, unzip):
    headers = response.headers
    res = []
    for k in sorted(headers.keys()):
        res.append(str((k, headers[k])) + '\n')
    res.append('\n')
    enc = 'utf8' if response.encoding is None else response.encoding
    if unzip:
        with zipfile.ZipFile(BytesIO(response.content)) as zfile:
            response_content = zfile.read(zfile.infolist()[0])
    else:
        response_content = response.content

    res.append(text_type(response_content, enc, 'ignore'))
    return ''.join(res)


CARRIED = frozenset(('host', 'port', 'scheme', 'auth', 'verify'))

NOT_CARRIED = frozenset(
    [
        'name', 'regexes', 'status_code', 'path', 'files', 'method', 'data',
        'tries', 'headers', 'unzip'])

KEYS = frozenset(CARRIED | NOT_CARRIED)


def run(script_str):
    try:
        reqs = eval(script_str)
    except SyntaxError:
        yield "Problem with script: " + traceback.format_exc()
        return

    s = requests.Session()

    defreq = {
        'host': 'localhost', 'port': 80, 'scheme': 'http', 'verify': False}
    failed = False
    for req in reqs:
        unrec = set(req.keys()) - KEYS
        if len(unrec) > 0:
            failed = True
            yield "The keys " + str(unrec) + " aren't recognized."
            break

        for k in CARRIED & set(req.keys()):
            defreq[k] = req[k]

        path = req.get('path', '/')
        url = defreq['scheme'] + '://' + defreq['host'] + ':' + \
            str(defreq['port']) + path

        auth = defreq.get('auth')
        verify = defreq['verify']

        method = req.get('method', 'get')

        try:
            yield "Name: " + req['name'] + '\n'
        except KeyError:
            pass

        try:
            files = dict(
                (k, open(v, 'rb')) for k, v in req['files'].items())
        except KeyError:
            files = None

        data = req.get('data')
        tries = req.get('tries', {'max': 1, 'period': 1})
        max_tries = tries.get('max', 10)
        period = tries.get('period', 1)
        headers = req.get('headers')
        unzip = req.get('unzip', False)
        allow_redirects = req.get('allow_redirects', False)

        j = 0
        failed = True
        msg = None
        while failed and j < max_tries:
            if j > 0:
                time.sleep(period)
            j += 1
            try:
                yield "Request: " + url + "\n"
                r = s.request(
                    method, url, files=files, data=data,
                    allow_redirects=allow_redirects, auth=auth, verify=verify,
                    headers=headers)
            except requests.exceptions.InvalidURL as e:
                msg = "Invalid URL: " + str(e) + '\n'
                break

            if 'status_code' in req:
                req_status_code = int(req['status_code'])
                if r.status_code != req_status_code:
                    msg = "The desired status code " + str(req_status_code) + \
                        " doesn't match the actual status code " + \
                        str(r.status_code) + ".\n" + response_str(r, unzip)
                    continue

            if 'regexes' in req:
                regex_failed = False
                k = 0
                while not regex_failed and k < len(req['regexes']):
                    pattern = req['regexes'][k]
                    k += 1
                    try:
                        if re.search(
                                pattern, response_str(r, unzip),
                                flags=re.MULTILINE | re.DOTALL) is None:
                            regex_failed = True
                    except sre_constants.error as e:
                        yield "Problem with regex: " + str(e) + '\n'
                        return
                if regex_failed:
                    msg = "The regular expression '" + pattern + \
                        "' fails.\n" + response_str(r, unzip)
                    continue

            failed = False

        if failed:
            yield msg
            break

    if not failed:
        yield "Passed all tests!\n"
        yield None



#!/usr/bin/env python

import versioneer
from setuptools import setup
versioneer.VCS = 'git'
versioneer.versionfile_source = 'imprimatur/_version.py'
versioneer.versionfile_build = 'imprimatur/_version.py'
versioneer.tag_prefix = ''
versioneer.parentdir_prefix = 'imprimatur-'

long_description = """\

Imprimatur
----------

Imprimatur is a tool for performing automated functional testing on web
applications. The tests are described in a simple test script. Along with the
standard HTTP methods, Imprimatur handles authentication, file uploads and
HTTPS. The responses are validated using regular expressions."""

cmdclass = dict(versioneer.get_cmdclass())

version = versioneer.get_version()

setup(
    name="imprimatur",
    version=version,
    cmdclass=cmdclass,
    description="Functional testing tool for web applications.",
    long_description=long_description,
    author="Tony Locke",
    author_email="tlocke@tlocke.org.uk",
    url="https://github.com/tlocke/imprimatur",
    license="GPL",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: Implementation",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Operating System :: OS Independent",
    ],
    keywords="functional testing web",
    packages=("imprimatur",),
    package_data={'imprimatur': ['templates/*.html']},
    install_requires=['requests==2.21.0', 'flask==1.0.2', 'six==1.10.0'],
    entry_points={'console_scripts': ['imprimatur = imprimatur.console:main']}
)



import imprimatur
import argparse
import sys


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "file",
        help="File containing the Imprimatur tests.",
        type=argparse.FileType())
    args = parser.parse_args()
    script_str = ''.join(args.file)

    for txt in imprimatur.run(script_str):
        if txt is None:
            exit(0)
        else:
            sys.stdout.write(txt)

    exit(1)
